#load required libraries
library(mlbench)
library(MASS)

#load Credit data
rm(list = ls())
Credit = read.csv("~/Downloads/Credit.csv")

# remove first column
Credit = Credit[,-1]

attach(Credit)

#set seed to ensure reproducible results
set.seed(42)
#split into training and test sets
Credit[,"train"] <- ifelse(runif(nrow(Credit))<0.8,1,0)

#separate training and test sets
trainset <- Credit[Credit$train==1,]
testset <- Credit[Credit$train==0,]

#get column index of train flag
trainColNum <- grep("train",names(trainset))
#remove train flag column from train and test sets
trainset <- trainset[,-trainColNum]
testset <- testset[,-trainColNum]

#get column index of predicted variable in dataset
typeColNum <- grep("Balance",names(Credit))

#build model
fit = lm(Balance ~ ., data = trainset)
summary(fit)

#predicted values (Balance)
predicted_values_lm <- predict.lm(fit, newdata = testset)
sum((Balance[Credit$train==0] - predicted_values_lm)^2)

library(leaps)
# perform feature selection
summary(regsubsets(Balance ~ ., trainset, method = 'exhaustive'))

#build model
fit = lm(Balance ~ Income+Limit+Cards+Student, data = trainset)
summary(fit)

#predicted values (Balance)
predicted_values_lm_4 <- predict.lm(fit, newdata = testset)
sum((Balance[Credit$train==0] - predicted_values_lm_4)^2)



# perform ridge
lambda = seq(0, 50000)
ridge = lm.ridge(Balance ~ ., trainset, lambda = lambda)
c = coef(ridge)


plot(lambda, c[,2], type='l', log = 'x', ylim = c(-30,30))
abline(h=0)
lines(lambda, c[,3], type='l')
lines(lambda, c[,4], type='l')
lines(lambda, c[,5], type='l')
lines(lambda, c[,6], type='l')
lines(lambda, c[,7], type='l')
lines(lambda, c[,8], type='l')
lines(lambda, c[,9], type='l')
lines(lambda, c[,10], type='l')
lines(lambda, c[,11], type='l')
lines(lambda, c[,12], type='l')

#load required library
library(glmnet)
#convert training data to matrix format
x <- model.matrix(Balance ~ ., trainset)
#convert class to numerical variable
y <- Balance[Credit$train==1]
# perform grid search to find optimal value of lambda
# alpha=1 => lasso
# check docs to explore other type.measure options
cv.out <- cv.glmnet(x,y,alpha=1,type.measure = "mse" )
#plot result
plot(cv.out)

#min value of lambda
lambda_min <- cv.out$lambda.min
#best value of lambda
lambda_1se <- cv.out$lambda.1se
#regression coefficients
coef(cv.out,s=lambda_1se)

#get test data
x_test <- model.matrix(Balance ~ ., testset)
#predicted values (Balance)
predicted_values_ridge <- predict(cv.out, newx = x_test, s=lambda_1se, type="response")
sum((Balance[Credit$train==0] - predicted_values_ridge)^2)

detach(Credit)
