#load required libraries
library(mlbench)
library(MASS)


#load Heart data
rm(list = ls())
heart = read.table("~/Downloads/SAheart.data", sep=",",head=T,row.names=1)
attach(heart)

#set seed to ensure reproducible results
set.seed(42)
#split into training and test sets
heart[,"train"] <- ifelse(runif(nrow(heart))<0.8,1,0)

#separate training and test sets
trainset <- heart[heart$train==1,]
testset <- heart[heart$train==0,]

#get column index of train flag
trainColNum <- grep("train",names(trainset))
#remove train flag column from train and test sets
trainset <- trainset[,-trainColNum]
testset <- testset[,-trainColNum]

#get column index of predicted variable in dataset
typeColNum <- grep("chd",names(heart))

#build model
glm_model <- glm(chd~.,data = trainset, family = binomial)
summary(glm_model)

#predict probabilities on testset
#type="response" gives probabilities, type="class" gives class
glm_prob <- predict.glm(glm_model,testset[,-typeColNum],type="response")

# try to evaluate the accuracy of our model
glm.pred = rep(0,dim(testset)[1])
glm.pred[glm_prob>.5]=1
chd.test = chd[heart$train==0]
table(chd.test, glm.pred)
mean(glm.pred==chd.test)


#load required library
library(glmnet)
#convert training data to matrix format
x <- model.matrix(chd~.,trainset)
#convert class to numerical variable
y <- trainset$chd
#perform grid search to find optimal value of lambda
#family= binomial => logistic regression, alpha=1 => lasso
# check docs to explore other type.measure options
cv.out <- cv.glmnet(x,y,alpha=1,family="binomial",type.measure = "mse" )
#plot result
plot(cv.out)

#min value of lambda
lambda_min <- cv.out$lambda.min
#best value of lambda
lambda_1se <- cv.out$lambda.1se
#regression coefficients
coef(cv.out,s=lambda_1se)

#get test data
x_test <- model.matrix(chd~.,testset)
#predict class, type="class"
lasso_prob <- predict(cv.out,newx = x_test,s=lambda_1se,type="response")

# try to evaluate the accuracy of our model
glm.pred = rep(0,dim(testset)[1])
glm.pred[lasso_prob>.5]=1
chd.test = chd[heart$train==0]
table(chd.test, glm.pred)
mean(glm.pred==chd.test)

detach(heart)